/*
 * Copyright (C) 2012 nico3000 - All Rights Reserved
 */
package actor;

import logging.Logger;
import gameapp.GameApp;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import util.XMLUtil;

/**
 * Erzeugt Instanzen der Klasse Actor und initialisiert sie.
 * @author nico3000
 */
public class ActorFactory {
    /**
     * Ungueltige Actor ID.
     */
    public static final int INVALID_ACTOR_ID = -1;
    
    /**
     * Letzte gueltige Actor ID.
     */
    private int lastActorId = 0;
    
    /**
     * Initialisierung der Instanz.
     * @return <tt>true</tt>, falls erfolgreich
     */
    public boolean init() {
        return ActorComponent.addComponentPackage("actor.components.");
    }
    
    /**
     * Erzeugt einen Actor aus einer XML Resource.
     * @param resource Die Resource des Actors
     * @param overrideData Moegliche Ueberschreibungen der Standarddaten
     * @return Der neue Actor, wenn erfolgreich; sonst null
     */
    public Actor createActor(String resource, Node overrideData) {
        Document doc = GameApp.loadXML(resource);
        Actor actor = null;
        if(doc != null) {
            String name = XMLUtil.getAttributeAsString(overrideData, "name");
            String parentActorName = XMLUtil.getAttributeAsString(overrideData, "parent");
            actor = new Actor(++this.lastActorId, name, parentActorName);
            Node actorNode = doc.getFirstChild();
            if(actor.init(actorNode, overrideData)) {
                Node componentNode = actorNode.getFirstChild();
                while(componentNode != null) {
                    if(componentNode.getNodeName().endsWith("Component")) {
                        ActorComponent component = ActorComponent.createComponent(componentNode.getNodeName());
                        if(component != null) {
                            component.setOwner(actor);
                            Node param = XMLUtil.getChildByType(overrideData, componentNode.getNodeName());
                            if(component.init((param != null ? param : componentNode).getFirstChild())) {
                                actor.addComponent(component);
                            }
                        }
                    } else if(!componentNode.getNodeName().equals("#text") && !componentNode.getNodeName().equals("#comment")) {
                        Logger.INSTANCE.error(componentNode.getNodeName() + " was ignored");
                    }
                    componentNode = componentNode.getNextSibling();
                }
                actor.postInit();
            } else {
                actor = null;
                --this.lastActorId;
            }
        }
        return actor;
    }
}
