/*
 * This software is the proprietary information of nico3000.
 */
package actor.components;

import actor.ActorComponent;
import event.EventData;
import event.EventListener;
import event.events.ActorMovedEvent;
import gameapp.GameApp;
import gameapp.audio.AudioBuffer;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import logging.Logger;
import org.w3c.dom.Node;
import util.XMLUtil;

/**
 *
 * @author nico3000
 */
public class AudioComponent extends ActorComponent implements EventListener {
    public static final int ID = 0xe2d69486;
    private final Map<String, AudioBuffer> samples = new HashMap<>();
    private final Set<AudioBuffer> autoPlays = new HashSet<>();

    @Override
    protected int getComponentId() {
        return ID;
    }

    @Override
    protected boolean init(Node data) {
        while(data != null) {
            switch(data.getNodeName()) {
                case "Sample":
                    String name = XMLUtil.getAttributeAsString(data, "name");
                    String resource = XMLUtil.getAttributeAsString(data, "resource");
                    if(name != null && resource != null) {
                        AudioBuffer buffer = new AudioBuffer(resource);
                        if(buffer.init()) {
                            this.samples.put(name, buffer);
                            this.transferProperties(buffer, data);
                        }
                    }
            }
            data = data.getNextSibling();
        }
        return true;
    }

    @Override
    protected void postInit() {
        GameApp.getEventManager().register(this, ActorMovedEvent.ID);
        for(AudioBuffer buffer : this.autoPlays) {
            buffer.play();
        }
        this.autoPlays.clear();
    }

    @Override
    protected void destroy() {
        for(AudioBuffer buffer : this.samples.values()) {
            buffer.destroy();
        }
        this.samples.clear();
        GameApp.getEventManager().unregister(this, ActorMovedEvent.ID);
    }
    
    public AudioBuffer getSample(String name) {
        return this.samples.get(name);
    }  
    
    private void transferProperties(AudioBuffer buffer, Node data) {
        Node property = data.getFirstChild();
        while(property != null) {
            if(property.getNodeName().equals("Property")) {
                String name = XMLUtil.getAttributeAsString(property, "name");
                if(name != null) {
                    switch(name) {
                        case "looping":
                            buffer.setLooping(XMLUtil.getAttributeAsBoolean(property, "value", false));
                            break;
                        case "autoplay":
                            if(XMLUtil.getAttributeAsBoolean(property, "value", false)) {
                                this.autoPlays.add(buffer);
                            }
                    }
                }
            }
            property = property.getNextSibling();
        }
    }

    @Override
    public void trigger(EventData data) {
        ActorMovedEvent evt = (ActorMovedEvent)data;
        if(evt.getActorId() == this.getOwner().getId()) {
            TransformComponent comp = (TransformComponent)this.getOwner().getComponent(TransformComponent.ID);
            if(comp != null) {
                for(AudioBuffer buffer : this.samples.values()) {
                    buffer.setPosition(comp.getPose().getPosition().x, comp.getPose().getPosition().y, comp.getPose().getPosition().z);
                }
            }
        }
    }
}
