/*
 * This software is the proprietary information of nico3000.
 */
package gameapp;

import gameapp.audio.AudioBuffer;
import java.nio.FloatBuffer;
import logging.Logger;
import org.lwjgl.BufferUtils;
import org.lwjgl.LWJGLException;
import org.lwjgl.openal.AL;
import org.lwjgl.openal.AL10;
import org.lwjgl.openal.ALCcontext;
import org.lwjgl.openal.ALCdevice;
import org.lwjgl.util.vector.Vector3f;

/**
 *
 * @author nico3000
 */
public class SoundLayer {
    private ALCdevice device;
    private ALCcontext context;
    private final FloatBuffer listener = BufferUtils.createFloatBuffer(9);
    
    public boolean init() {
        try {
            AL.create();
            this.device = AL.getDevice();
            this.context = AL.getContext();
        } catch (LWJGLException ex) {
            Logger.INSTANCE.error(ex.getMessage());
            return false;
        }
        return true;
    }
    
    public void test() {
        AudioBuffer audio = new AudioBuffer("Sounds/SomeNights.wav");
        audio.init();
        audio.setLooping(true);
        audio.play();
    }
    
    public void destroy() {
        AL.destroy();
    }

    public ALCdevice getDevice() {
        return this.device;
    }

    public ALCcontext getContext() {
        return this.context;
    }
    
    public void setListener(Vector3f pos, Vector3f viewDir, Vector3f upDir) {
        upDir.scale(-1.0f);
        this.listener.position(0);
        this.listener.limit(9);
        pos.store(this.listener);
        viewDir.store(this.listener);
        upDir.store(this.listener);
        this.listener.position(3);
        AL10.alListener(AL10.AL_ORIENTATION, this.listener);
        checkError();
        this.listener.position(0);
        this.listener.limit(3);
        AL10.alListener(AL10.AL_POSITION, this.listener);
        checkError();
    }
    
    public boolean checkError() {
        int error = AL10.alGetError();
        switch(error) {
            case AL10.AL_NO_ERROR:
                return true;
            case AL10.AL_INVALID_ENUM:
                Logger.INSTANCE.error(0, "AL_INVALID_ENUM");
                return false;
            case AL10.AL_INVALID_NAME:
                Logger.INSTANCE.error("AL_INVALID_NAME");
                return false;
            case AL10.AL_INVALID_OPERATION:
                Logger.INSTANCE.error("AL_INVALID_OPERATION");
                return false;
            case AL10.AL_INVALID_VALUE:
                Logger.INSTANCE.error("AL_INVALID_VALUE");
                return false;
            case AL10.AL_OUT_OF_MEMORY:
                Logger.INSTANCE.error("AL_OUT_OF_MEMORY");
                return false;
            default:
                Logger.INSTANCE.error("Unknown AL error (%d).", Integer.toHexString(error));
                return false;
        }
    }
}
