/*
 * This software is the proprietary information of nico3000.
 */
package gui.components;

/**
 * Ein Label ist eine Komponente, die nur Text oder ein Bild anzeigt und
 * ansonsten keine Funktionalitaet bietet.
 * @author nico3000
 */
public final class Label extends Component {
    /**
     * Der Text des Labels.
     */
    private String text;
    /**
     * Gibt an, ob ein Rahmen fuer das Label dargestellt werden soll.
     */
    private boolean decorated;

    /**
     * Konstruktor.
     * @param text Der Text des Labels
     * @param decorated Falls <tt>true</tt> wird ein Rahmen um das Label
     *                  gezeichnet
     */
    public Label(String text, boolean decorated) {
        super(REPAINT_ON_RESIZE);
        this.text = text;
        this.setDecorated(decorated);
    }

    /**
     * Setzt die Sichtbarkeit eines Rahmens.
     * @param decorated Falls <tt>true</tt> wird ein Rahmen um das Label
     *                  gezeichnet
     */
    public void setDecorated(boolean decorated) {
        if(this.decorated != decorated) {
            this.decorated = decorated;
            this.needsRepaint();
            if(this.decorated) {
                this.setRepaintBit(REPAINT_ON_NULL_TEXTURE);
            } else {
                this.clearRepaintBit(REPAINT_ON_NULL_TEXTURE);
            }
        }
    }

    /**
     * Setzt den Text des Labels.
     * @param text Der neue Text
     */
    public void setText(String text) {
        this.text = text;
        this.needsRepaint();
    }

    /**
     * Liefert den aktuellen Text.
     * @return Der aktuelle Text
     */
    public String getText() {
        return text;
    }
    
    @Override
    protected void repaint() {
        this.setTexture(this.getGui().getDrawer().createLabel(this.text, this.getWidth(), this.getHeight(), this.decorated));
    }    
}
