/*
 * This software is the proprietary information of nico3000.
 */
package gui.components;

import graphics.texture.Texture;
import gui.Gui;
import static gui.components.Component.REPAINT_ON_NULL_TEXTURE;
import util.MathUtil;

/**
 *
 * @author nico3000
 */
public class TextArea extends Component {
    private String text = "";
    private int scrolled = 0;
    private int scrollMax = 0;
    private boolean buttonDown = false;
    private final Component textComponent = new Component(REPAINT_ON_RESIZE) {
        @Override
        protected void repaint() {
            // nothing to do
        }

        @Override
        public boolean isGhost() {
            return true;
        }
    };
    private final Scrollbar scrollbar = new Scrollbar(this.textComponent);

    public TextArea(String text) {
        super(REPAINT_ON_NULL_TEXTURE | REPAINT_ON_RESIZE);
        this.text = text;
        this.textComponent.setLocalZOrder(1);
        this.scrollbar.setLocalZOrder(2);
    }

    public void setText(String text) {
        this.text = text;
    }

    @Override
    public void onCursorMoved(int posX, int posY, int dx, int dy) {
        if(this.buttonDown) {
            this.scroll(dy);
        }
    }

    @Override
    public void onButtonDown(int button, int x, int y) {
        if(button == 0) {
            this.buttonDown = true;
        }
    }

    @Override
    public void onButtonUp(int button, int x, int y, boolean clicked) {
        if(button == 0) {
            this.buttonDown = false;
        }
    }

    @Override
    protected void repaint() {
        //Pair<Texture, Integer> pair = this.getGui().getDrawer().createTextArea(this.text, this.getWidth(), this.getHeight(), this.scrolled);
        //this.setTexture(pair.left);
        //this.scrollMax = pair.right;
        this.setTexture(this.getGui().getDrawer().createLabel(null, this.getWidth(), this.getHeight(), true));
        this.scroll(0);
    }

    @Override
    public void setSize(int width, int height) {
        super.setSize(width, height);
        this.updateTextComponent();        
    }

    @Override
    public boolean onWheelMoved(int delta) {
        this.scroll(delta / 10);
        return true;
    }

    @Override
    public boolean init(Gui gui) {
        if(!super.init(gui)) {
            return false;
        } else {
            this.textComponent.setId(this.getId() + "_textarea");
            if(gui.addComponent(this.textComponent)) {
                this.addChild(this.textComponent);
                this.setText(this.text);
                this.scrollbar.setId(this.getId() + "_scrollbar");
                if(gui.addComponent(this.scrollbar)) {
                    this.addChild(this.scrollbar);
                    //this.scrollbar.setVisible(false);
                    return true;
                } else {
                    gui.removeComponent(this.textComponent);
                    this.destroy();
                    return false;
                }
            } else {
                this.destroy();
                return false;
            }            
        }
    }
    
    public void scroll(int delta) {
        this.scrolled = MathUtil.clamp(this.scrolled + delta, 0, this.scrollMax);
        this.textComponent.setPosition(Component.REF_LEFT | Component.REF_TOP, 0, this.getHeight() + this.scrolled);
    }   
    
    private void updateTextComponent() {
        this.scrollbar.setSize(16, this.getHeight());
        this.scrollbar.setPosition(this.getWidth() - 16, 0);
        Texture texture = this.getGui().getDrawer().drawTextBlock(this.text, this.getWidth() - 16);
        this.scrollMax = texture.getHeight() - this.getHeight();
        this.textComponent.setSize(texture.getWidth(), texture.getHeight());
        this.textComponent.setTexture(texture);
    }
}
