/*
 * This software is the proprietary information of nico3000.
 */
package main;

import actor.Actor;
import actor.components.AudioComponent;
import gameapp.GameApp;
import gameapp.audio.AudioBuffer;
import logging.Logger;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import util.XMLUtil;

/**
 *
 * @author nico3000
 */
public class ExampleGameLogic extends BaseGameLogic {

    @Override
    public void loadLevel(String resource) {
        Document doc = GameApp.loadXML(resource);
        Node node = XMLUtil.getChildByType(doc, "Scene");
        if(node != null) {
            node = node.getFirstChild();
        }
        while(node != null) {
            switch(node.getNodeName()) {
                case "Actor":
                    this.createActor(node);
                    break;
                case "#comment":
                case "#text":
                    // nothing to do
                    break;
                default:
                    Logger.INSTANCE.warning("node " + node.getNodeName() + " was ignored in " + resource);
                    break;
            }
            node = node.getNextSibling();
        }
    }
    
    /**
     * Erzeugt einen Actor aus einer Node einer xml-Leveldatei.
     * @param node Die Node des Actors
     */
    private void createActor(Node node) {
        String resource = XMLUtil.getAttributeAsString(node, "resource");
        if(resource != null) {
            this.createActor(resource, node);
        }
    } 
    
    public void playSound(String identifier) {
        AudioBuffer sample = this.getSample(identifier);
        if(sample != null) {
            sample.play();
        }
    }
    
    public void pauseSound(String identifier) {
        AudioBuffer sample = this.getSample(identifier);
        if(sample != null) {
            sample.pause();
        }
    }
    
    public void stopSound(String identifier) {
        AudioBuffer sample = this.getSample(identifier);
        if(sample != null) {
            sample.stop();
        }
    }
    
    private AudioBuffer getSample(String identifier) {
        String split[] = identifier.split(".", 2);
        if(split.length == 2) {
            Actor actor = this.getActor(split[0]);
            if(actor != null) {
                AudioComponent comp = (AudioComponent)actor.getComponent(AudioComponent.ID);
                if(comp != null) {
                    AudioBuffer sample = comp.getSample(split[1]);
                    if(sample != null) {
                        return sample;
                    } else {
                        Logger.INSTANCE.warning("AudioComponent of actor %s does not have a sample %s.", actor.getName(), split[1]);
                    }
                } else {
                    Logger.INSTANCE.warning("Actor %s does not have an AudioComponent.", actor.getName());
                }
            }            
        } else {
            Logger.INSTANCE.warning("Illegal identifier %s.", identifier);
        }
        return null;
    }
}
