/*
 * This software is the proprietary information of nico3000.
 */
package n3gui.components;

import graphics.texture.Texture;
import graphics.texture.TextureLoader;
import gameapp.GameApp;
import n3gui.Component;
import n3gui.Dimension;
import org.lwjgl.input.Mouse;
import util.MathUtil;

/**
 *
 * @author nico3000
 */
public class Cursor extends Component {
    private int offsetX = 2;
    private int offsetY = 2;
    
    /**
     * Konstruktor.
     */
    public Cursor() {
        Mouse.setGrabbed(true);
        this.setRepaintBits(REPAINT_ON_NULL_TEXTURE);
    }

    @Override
    protected void repaint() {
        Texture tex = TextureLoader.getTexture("Textures/Cursor.png");
        this.setTexture(tex);
        tex.release();
    }
    
    /**
     * Liefert die x-Koordinate der Cursorposition.
     * @return Die Cursor x-Koordinate
     */
    public int getCursorX() {
        return this.getNode().getDimension().getGlobalX() + this.offsetX;
    }
    
    /**
     * Liefert die y-Koordinate der Cursorposition
     * @return Die Cursor y-Koordinate
     */
    public int getCursorY() {
        return this.getNode().getDimension().getGlobalY() + this.offsetY;
    }

    /**
     * Bewegt den Cursor um eine bestimmte Distanz. Die neue Position wird dabei
     * innerhalb der Aufloesung des Anwendungsfensters geclampt.
     * @param dX Die Bewegung des Cursors in x-Richtung
     * @param dY Die Bewegung des Cursors in y-Richtung
     */
    public void moveCursor(int dX, int dY) {
        int newCursorX = MathUtil.clamp(this.getCursorX() + dX, 0, GameApp.getGraphicsLayer().getWidth());
        int newCursorY = MathUtil.clamp(this.getCursorY() + dY, 0, GameApp.getGraphicsLayer().getHeight());
        this.getNode().getDimension().setPosition(newCursorX - this.offsetX, newCursorY - this.offsetY);
    }

    @Override
    protected int getPreferredWidth() {
        return 16;
    }

    @Override
    protected int getPreferredHeight() {
        return 16;
    }

    @Override
    protected void onInitialized() {
        this.getNode().getDimension().setReferencePoint(Dimension.REF_H_L | Dimension.REF_V_T);
        this.getNode().setAlwaysOnTop(true);
    }

    @Override
    public boolean isFocusable() {
        return false;
    }

    @Override
    public boolean isHoverable() {
        return false;
    }
}
