/*
 * This software is the proprietary information of nico3000.
 */
package n3gui.components;

import graphics.texture.Texture;
import graphics.texture.TextureLoader;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.image.BufferedImage;
import n3gui.Component;
import n3gui.DrawHelper;

/**
 *
 * @author nico3000
 */
public class Label extends Component {
    /**
     * Der Text des Labels.
     */
    private String text = "";

    /**
     * Konstruktor.
     * @param text Der Text des Labels
     */
    public Label(String text) {
        this.setRepaintBits(REPAINT_ON_RESIZE | REPAINT_ON_NULL_TEXTURE);
        this.text = text;
    }

    /**
     * Liefert den Text des Labels.
     * @return Der Text
     */
    public String getText() {
        return this.text;
    }

    /**
     * Setzt den Text des Labels.
     * @param text Der neue Text
     */
    public void setText(String text) {
        this.text = text;
        this.mustRepaint();
    }

    @Override
    protected void repaint() {
        int width = this.getNode().getDimension().getWidth();
        int height = this.getNode().getDimension().getHeight();
        DrawHelper helper = this.getNode().getTree().getDrawHelper();
        BufferedImage image = helper.createBaseImage(width, height);
        Graphics2D graphics = image.createGraphics();
        helper.drawDecoration(graphics, new Rectangle(0, 0, width - 1, height - 1));
        graphics.setColor(helper.getDefaultColor(DrawHelper.ColorType.FONT));
        graphics.setFont(helper.getDefaultFont());
        helper.drawText(this.text, graphics, new Rectangle(6, 6, width - 12, height - 12), DrawHelper.ALIGN_H_C | DrawHelper.ALIGN_V_C, 1);
        Texture tex = TextureLoader.createTexture(image);
        this.setTexture(tex);
        tex.release();
    }
    
}
