/*
 * This software is the proprietary information of nico3000.
 */
package n3gui.components;

import graphics.texture.Texture;
import graphics.texture.TextureLoader;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.image.BufferedImage;
import logging.Logger;
import n3gui.Component;
import n3gui.Dimension;
import n3gui.DrawHelper;
import n3gui.GuiAction;

/**
 *
 * @author nico3000
 */
public class Window extends Component {
    /**
     * Der Titel des Fensters.
     */
    private String title;
    /**
     * Rechteck, das die Titelleiste enthaelt.
     */
    private Rectangle titleRect = new Rectangle();
    /**
     * Gibt an, ob sich der Cursor mit gedrueckter linker Maustaste ueber der
     * Titelleiste befindet.
     */
    private boolean grabbed = false;
    
    /**
     * Konstruktor.
     * @param title Der Titel des Fensters 
     */
    public Window(String title) {
        this.setRepaintBits(REPAINT_ON_RESIZE | REPAINT_ON_FOCUS_CHANGED | REPAINT_ON_NULL_TEXTURE);
        this.title = title;
        this.setContentPadding(8, 40, 8, 8);
    }

    /**
     * Setzt den Titel des Fensters.
     * @param title Der neue Titel
     */
    public void setTitle(String title) {
        this.title = title;
        this.mustRepaint();
    }

    @Override
    protected void onInitialized() {
        Button btn = new Button("res:Textures/N3Gui/Close.png", Button.ButtonColor.RED);
        btn.setAction(new GuiAction() {
            @Override
            public void fire() {
                Window.this.getNode().setVisible(false);
            }
        });
        btn.setClipOffset(1);
        Dimension dim = this.getNode().getTree().addComponent(this.getNode().getQualifier(), null, btn);
        if(dim == null) {
            Logger.INSTANCE.warning("Initialization of close button for %s failed.", this);
        } else {
            dim.setReferencePoint(Dimension.REF_H_R | Dimension.REF_V_T);
            dim.setPosition(-8, 4);
            dim.setSize(24, 24);
        }
    }

    @Override
    protected void onCursorMoved(int cursorX, int cursorY, int dX, int dY) {
        if(this.grabbed) {
            this.getNode().getDimension().move(dX, dY);
            
        }
    }

    @Override
    protected void onButtonDown(int button) {
        if(button == 0) {
            if(this.containsCursor(this.titleRect)) {
                this.grabbed = true;
            }
        }
    }

    @Override
    protected void onButtonUp(int button) {
        if(button == 0) {
            this.grabbed = false;
        }
    }

    @Override
    protected void onSizeChanged() {
        this.titleRect.setBounds(24, 0, this.getNode().getDimension().getWidth() - 28, 32);
    }

    @Override
    protected void repaint() {
        int width = this.getNode().getDimension().getWidth();
        int height = this.getNode().getDimension().getHeight();
        BufferedImage image = this.getNode().getTree().getDrawHelper().createBaseImage(width, height);
        Graphics2D graphics = image.createGraphics();
        this.getNode().getTree().getDrawHelper().drawDecoration(graphics, new Rectangle(0, 0, width - 1, height - 1));
        if(this.title != null) {
            graphics.setColor(this.getNode().getTree().getDrawHelper().getDefaultColor(DrawHelper.ColorType.FONT));
            graphics.setFont(this.getNode().getTree().getDrawHelper().getDefaultFont().deriveFont(Font.BOLD));
            this.getNode().getTree().getDrawHelper().drawText(this.title, graphics, this.titleRect, DrawHelper.ALIGN_H_L | DrawHelper.ALIGN_V_C, 1);
            //graphics.drawRect(this.titleRect.x, this.titleRect.y, this.titleRect.width, this.titleRect.height);
            graphics.drawLine(8, this.titleRect.height, width - 8, this.titleRect.height);
        }
        Texture texture = TextureLoader.createTexture(image);
        this.setTexture(texture);
        texture.release();
    }    
}
