/*
 * This software is the proprietary information of nico3000.
 */
package util;

import java.util.HashMap;
import java.util.Map;

/**
 * Mithilfe dieser Klasse koennen zwei Objekte miteinander verknuepft werden.
 * Dabei kann jedes Objekt mit maximal einem anderen verknuepft sein.
 * @author nico3000
 */
public class Joiner<A, B> {
    /**
     * Die normale HashMap.
     */
    private final Map<A, B> mappingsA2B = new HashMap<>();
    /**
     * Eine umgedrehte HashMap.
     */
    private final Map<B, A> mappingsB2A = new HashMap<>();
    
    /**
     * Verknuepft zwei Objekte miteinander. Eventuell bestehende Verknuepfungen
     * dieser Objekte mit anderen werden aufgehoben.
     * @param a Das erste Objekt
     * @param b Das zweite Objekt
     */
    public void join(A a, B b) {
        if(this.mappingsB2A.containsKey(b)) {
            this.mappingsB2A.remove(b);
        }
        this.mappingsA2B.put(a, b);
        this.mappingsB2A.put(b, a);
    }
    
    /**
     * Liefert das mit einem Objekt verknuepfte andere Objekt.
     * @param a Das Objekt
     * @return Das veknuepfte andere Objekt
     */
    public B getJoined1(A a) {
        return this.mappingsA2B.get(a);
    }
    
    /**
     * Liefert das mit einem Objekt verknuepfte andere Objekt.
     * @param b Das Objekt
     * @return Das veknuepfte andere Objekt
     */    
    public A getJoined2(B b) {
        return this.mappingsB2A.get(b);
    }
}
