/*
 * This software is the proprietary information of nico3000.
 */
package view;

/**
 * Ein ScreenElement ist eine eine Einheit, die auf dem Bildschirm angezeigt
 * werden kann, wie etwa die Szene selbst, ein Menu oder ein einzelner Button.
 * Es besitzt eine Tiefe, sodass die Menge der Elemente angeordnet werden
 * koennen und es kann sichtbar oder versteckt sein.
 * @author nico3000
 */
public abstract class ScreenElement implements Comparable<ScreenElement> {
    /**
     * Die Sichtbarkeit des Elements.
     */
    private boolean visible = true;
    /**
     * Die z-Ebene des Elements.
     */
    private int zOrder = 0;

    /**
     * Liefert die aktuelle Tiefe.
     * @return Die Tiefe
     */
    public int getzOrder() {
        return zOrder;
    }

    /**
     * Setzt eine neue Tiefe.
     * @param zOrder Die neue Tiefe
     */
    public void setzOrder(int zOrder) {
        this.zOrder = zOrder;
    }

    /**
     * Liefert die Sichtbarkeit des Elements.
     * @return <tt>true</tt>, falls sichtbar
     */
    public boolean isVisible() {
        return visible;
    }

    /**
     * Setzt die Sichtbarkeit des Elements.
     * @param visible Die neue Sichtbarkeit
     */
    public void setVisible(boolean visible) {
        this.visible = visible;
    }
    
    /**
     * Wird aufgerufen, wenn das Graphicsdevice wiederhergestellt und nachdem
     * das Element erfolgreich der <tt>HumanView</tt> hinzugefuegt wurde, um das
     * Element zu re- oder initialisieren.
     * @return <tt>true</tt>, falls erfolgreich
     */
    public abstract boolean onRestore();
    
    /**
     * Zerstoert das Element.
     */
    public abstract void onDestroy();
    
    /**
     * Wird aufgerufen, wenn das Graphicsdevice verloren geht. <i>Hinweis</i>:
     * Uns ist nicht bekannt, wann das jemals geschieht.
     * @return <tt>true</tt>, wenn erfolgreich
     */
    public abstract boolean onLostDevice();
    
    /**
     * Rendert das Element.
     */
    public abstract void onRender();
    
    /**
     * Aktualisiert das Element.
     * @param deltaMillis Die Anzahl der Millieskunden, die seit dem letzten
     *                    Aufruf vergangen ist
     */
    public abstract void onUpdate(long deltaMillis);

    @Override
    public int compareTo(ScreenElement o) {
        return o == null ? -1 : Integer.compare(this.zOrder, o.zOrder);
    }
}
